VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsProcesses"
Attribute VB_GlobalNameSpace = True
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'---------------------------------------------------------------------------------------
' Module    : clsProcesses
' DateTime  : 30/04/2007 23:21
' Author    : Vb System Library
' Purpose   : Gerer les processus
' Copyright : (C) 2007 L'quipe Vb System Library
'              See "modPresentation" for more information
'---------------------------------------------------------------------------------------
' Librairie de fonctions permettant d'agir simplement et rapidement sur le systme.
' Copyright (C) 2007 - L'quipe Vb System Library
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the Free Software
' Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
'---------------------------------------------------------------------------------------


Option Explicit

' Les informations sur un thread
Public Type ThreadInfo
    ID                  As Long
    ParentProcessID     As Long
    StartAddress        As Long
    Priority            As Long
    BasePriority        As Long
    ContextSwitchCount  As Long
    State               As Long
    WaitReason          As Long
    WaitTime            As Long
    KernelTime          As Currency
    UserTime            As Currency
    CreateTime          As Currency
End Type

' Les informations sur un processus
Public Type ProcessInfo
    ProcessName         As String
    ProcessID           As Long
    ParentProcessID     As Long
    ImageFile           As String
    UserName            As String
    ThreadCount         As Long
    HandleCount         As Long
    PriorityClass       As Long
    PagefileUsage       As Long
    CommandLine         As String
    CurrentDirectory    As String
    Thread()            As ThreadInfo
End Type

' Les informations basiques sur un processus (informations statiques dans le temps)
Public Type ProcessBasicInformations
    ProcessID           As Long
    ProcessName         As String
    ParentProcessID     As Long
    ImageFile           As String
    UserName            As String
    CommandLine         As String
    CurrentDirectory    As String
    PEBAdress           As Long
End Type

' Les informations sur l'utilisation mmoire d'un processus
Public Type PROCESS_MEMORY_COUNTERS
    cb                          As Long ' Taille de la structure
    PageFaultCount              As Long ' Nombre d'erreurs de page
    PeakWorkingSetSize          As Long ' Utilisation max de la mmoire RAM
    WorkingSetSize              As Long ' Utilisation de la mmoire RAM
    QuotaPeakPagedPoolUsage     As Long ' Utilisation max de la mmoire pagine
    QuotaPagedPoolUsage         As Long ' Utilisation de la mmoire pagine
    QuotaPeakNonPagedPoolUsage  As Long
    QuotaNonPagedPoolUsage      As Long
    PagefileUsage               As Long  ' Utilisation de la mmoire virtuelle
    PeakPagefileUsage           As Long ' Utilisation max de la mmoire virtuelle
End Type

' Les informations sur les temps processeur du processus
Public Type ProcessorTimes
    KernelTime      As Double ' Temps noyau
    UserTime        As Double ' Temps utilisateur
    ProcessorTime   As Double ' Temps processeur
End Type

' Enum pour les priorits du processus
Public Enum ProcessPriority
    [Idle] = IDLE_PRIORITY_CLASS
    [Below Normal] = BELOW_NORMAL_PRIORITY_CLASS
    [Normal] = NORMAL_PRIORITY_CLASS
    [Above Normal] = ABOVE_NORMAL_PRIORITY_CLASS
    [High] = HIGH_PRIORITY_CLASS
    [Real Time] = REALTIME_PRIORITY_CLASS
End Enum

'cre un buffer de processus
Private Function CreateProcessesBuffer(ByRef Buffer() As Byte) As Long

    Dim Length      As Long
    Dim RetLength   As Long
    
    Length = &H1000& 'longueur minimale
    ReDim Buffer(Length)
    
    Do While NtQuerySystemInformation(SystemProcessesAndThreadsInformation, ByVal VarPtr(Buffer(0)), Length, RetLength) = STATUS_INFO_LENGTH_MISMATCH
        Length = Length * 2
        ReDim Buffer(Length)
    Loop
    
    ReDim Preserve Buffer(RetLength)
    CreateProcessesBuffer = RetLength
    
End Function

'remplit les tableaux avec les donnes
Private Sub FillProcArray(ByRef pArray As SafeArray1d, ByVal Address As Long, ByRef Process() As SYSTEM_PROCESSES)

    With pArray
        .cbElements = 184
        .cDims = 1
        .cElements = 1
        .fFeatures = FADF_AUTO Or FADF_FIXEDSIZE
        .lLbound = 0
        .pvData = Address
    End With
    
    CopyMemory ByVal VarPtrArray(Process), VarPtr(pArray), 4

End Sub

Private Sub FillThreadArray(ByRef tArray As SafeArray1d, ByVal Address As Long, ByRef Proc() As SYSTEM_PROCESSES)

    With tArray
        .cbElements = 60
        .cDims = 1
        .cElements = Proc(0).ThreadCount
        .fFeatures = FADF_AUTO Or FADF_FIXEDSIZE
        .lLbound = 0
        .pvData = Address + 184
    End With
    
    CopyMemory ByVal VarPtrArray(Proc(0).Threads), VarPtr(tArray), 4
    
End Sub


'---------------------------------------------------------------------------------------
' Procedure : GetProcessByID
' DateTime  : 01/05/2007 00:08
' Author    : Vb System Library
' Purpose   : Renvoie un objet Processus
' Params    : ProcessID : ID du processus
'             [Optionnel] RefreshInfos : Si = TRUE rafraichit les informations du processus
'                                           = FALSE par dfaut
'---------------------------------------------------------------------------------------
Public Function GetProcessByID(ByVal ProcessID As Long, Optional ByVal RefreshInfos As Boolean = False) As Process
    Set GetProcessByID = New Process
    GetProcessByID.GetProcessByID ProcessID, RefreshInfos
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessList
' DateTime  : 02/12/2006 19:14
' Author    : Vb System Library
' Purpose   : Enumere les processus en cours
' Params    : ProcessInfos() Tableau contenant les infos des processus
'               Tableau allant de 0  (Nb de processus - 1)
'---------------------------------------------------------------------------------------
Public Function GetProcessList(ByRef ProcessLst() As ProcessInfo) As Long

    Dim ArrProcess      As SafeArray1d
    Dim ArrThread       As SafeArray1d
    Dim Ret             As Long
    Dim Buffer()        As Byte
    Dim Process()       As SYSTEM_PROCESSES
    Dim NextProcess     As Long
    Dim x               As Long
    
    Ret = CreateProcessesBuffer(Buffer)
    If Ret = 0 Then Exit Function
    
    FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
    FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process
    ReDim ProcessLst(x) As ProcessInfo
    
    With ProcessLst(x)
        .ProcessName = "[System Process]"
        .ThreadCount = 0
        .ParentProcessID = 0
        .PagefileUsage = Process(0).VmCounters.WorkingSetSize
    End With
    
    
    NextProcess = Process(0).NextEntryDelta
    
    Do
        x = x + 1
        FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
        FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process
        ReDim Preserve ProcessLst(x) As ProcessInfo
        GetProcessFullInformation Process(0), ProcessLst(x), ProcessLst(x).Thread
        GetThreadFullInformation Process(0), ProcessLst(x).Thread
        If Process(0).NextEntryDelta = 0 Then Exit Do
        NextProcess = NextProcess + Process(0).NextEntryDelta
    Loop
        
    GetProcessList = x + 1
    
    Erase Process
    Erase Buffer
    
    
    With ArrProcess
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
    With ArrThread
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
            
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessIDList
' DateTime  : 01/12/2006 20:17
' Author    : Vb System Library
' Purpose   : Renvoie la liste des ID des processus ouverts
' Params    : Parametre de retour : PidList (tableau de Long)
'               Le tableau s'tend de 0  (Nb de processus - 1)
'               La fonction renvoie le nombre de processus trouvs
'---------------------------------------------------------------------------------------
Public Function GetProcessIDList(ByRef listProcessID() As Long) As Long
    
    Dim ArrProcess      As SafeArray1d
    Dim Ret             As Long
    Dim Buffer()        As Byte
    Dim Process()       As SYSTEM_PROCESSES
    Dim NextProcess     As Long
    Dim x               As Long
    
    Ret = CreateProcessesBuffer(Buffer)
    If Ret = 0 Then Exit Function
    
    FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
    ReDim listProcessID(x) As Long
    
    listProcessID(x) = Process(0).ProcessID
    NextProcess = Process(0).NextEntryDelta
    
    Do
        x = x + 1
        FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
        ReDim Preserve listProcessID(x) As Long
        listProcessID(x) = Process(0).ProcessID
        If Process(0).NextEntryDelta = 0 Then Exit Do
        NextProcess = NextProcess + Process(0).NextEntryDelta
    Loop
    
    GetProcessIDList = x + 1
    
    Erase Process
    Erase Buffer
    
    With ArrProcess
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessBasicInfos
' DateTime  : 02/12/2006 19:14
' Author    : Vb System Library
' Purpose   : Renvoie les infos statiques d'un processus
' Params    : Renvoie une variable de type ProcessBasicInformations
'---------------------------------------------------------------------------------------
Public Function GetProcessBasicInfos(ByVal ProcessID As Long) As ProcessBasicInformations

    Dim ArrProcess      As SafeArray1d
    Dim Ret             As Long
    Dim Buffer()        As Byte
    Dim Process()       As SYSTEM_PROCESSES
    Dim NextProcess     As Long
    Dim x               As Long
    
    Ret = CreateProcessesBuffer(Buffer)
    If Ret = 0 Then Exit Function
    
    FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
    
    If Process(0).ProcessID = ProcessID Then
        With GetProcessBasicInfos
            .ProcessName = "[System Process]"
            .ParentProcessID = 0
        End With
    Else
        NextProcess = Process(0).NextEntryDelta
        Do
            FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
            If Process(0).ProcessID = ProcessID Then
                GetProcessBasicInformation Process(0), GetProcessBasicInfos
                Exit Do
            End If
            If Process(0).NextEntryDelta = 0 Then Exit Do
            NextProcess = NextProcess + Process(0).NextEntryDelta
        Loop
    End If
    
    Erase Process
    Erase Buffer
    
    With ArrProcess
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessInfos
' DateTime  : 02/12/2006 19:14
' Author    : Vb System Library
' Purpose   : Renvoie les infos d'un processus
' Params    : Renvoie une variable de type ProcessInfo
'---------------------------------------------------------------------------------------
Public Function GetProcessInfos(ByVal ProcessID As Long) As ProcessInfo

    Dim ArrProcess      As SafeArray1d
    Dim ArrThread       As SafeArray1d
    Dim Ret             As Long
    Dim Buffer()        As Byte
    Dim Process()       As SYSTEM_PROCESSES
    Dim NextProcess     As Long
    Dim x               As Long
    
    Ret = CreateProcessesBuffer(Buffer)
    If Ret = 0 Then Exit Function
    
    FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
    FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process

    If Process(0).ProcessID = ProcessID Then
        With GetProcessInfos
            .ProcessName = "[System Process]"
            .ThreadCount = 0
            .ParentProcessID = 0
            .PagefileUsage = Process(0).VmCounters.WorkingSetSize
        End With
    Else
        NextProcess = Process(0).NextEntryDelta
        Do
            FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
            FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process
            If Process(0).ProcessID = ProcessID Then
                GetProcessFullInformation Process(0), GetProcessInfos, GetProcessInfos.Thread
                GetThreadFullInformation Process(0), GetProcessInfos.Thread
                Exit Do
            End If
            If Process(0).NextEntryDelta = 0 Then Exit Do
            NextProcess = NextProcess + Process(0).NextEntryDelta
        Loop
    End If
    
    Erase Process
    Erase Buffer
    
    
    With ArrProcess
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
    With ArrThread
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
        
End Function

'---------------------------------------------------------------------------------------
' Procedure : GetProcessThreads
' DateTime  : 02/12/2006 19:14
' Author    : Vb System Library
' Purpose   : Renvoie la liste des thread d'un processus
' Params    : Renvoie un tableau de type ThreadInfo
'---------------------------------------------------------------------------------------
Public Function GetProcessThreads(ByVal ProcessID As Long) As ThreadInfo()

    Dim ArrProcess      As SafeArray1d
    Dim ArrThread       As SafeArray1d
    Dim Ret             As Long
    Dim Buffer()        As Byte
    Dim Process()       As SYSTEM_PROCESSES
    Dim NextProcess     As Long
    Dim x               As Long
    Dim ThInfos()       As ThreadInfo
    
    Ret = CreateProcessesBuffer(Buffer)
    If Ret = 0 Then Exit Function
    
    FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
    FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process

    If Process(0).ProcessID = ProcessID Then
        ReDim ThInfos(0)
        ThInfos(0).ParentProcessID = ProcessID
    Else
        NextProcess = Process(0).NextEntryDelta
        Do
            FillProcArray ArrProcess, VarPtr(Buffer(NextProcess)), Process
            If Process(0).ProcessID = ProcessID Then
                FillThreadArray ArrThread, VarPtr(Buffer(NextProcess)), Process
                GetThreadFullInformation Process(0), ThInfos
                Exit Do
            End If
            If Process(0).NextEntryDelta = 0 Then Exit Do
            NextProcess = NextProcess + Process(0).NextEntryDelta
        Loop
    End If
    
    GetProcessThreads = ThInfos

    Erase Process
    Erase Buffer
    
    With ArrProcess
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
    With ArrThread
        .cbElements = 0
        .cDims = 0
        .pvData = 0
    End With
    
End Function

'---------------------------------------------------------------------------------------
' Procedure : IsProcessRunning
' DateTime  : 23/11/2006 15:36
' Author    : Vb System Library
' Purpose   : Renvoie TRUE si le processus est toujours ouvert, sinon FALSE
' Params    : pid : ID du processus
'---------------------------------------------------------------------------------------
Public Function IsProcessRunning(ByVal PID As Long) As Boolean
    
    Dim hProcess As Long
    Dim lngExit As Long
    
    If PID = 0 Then IsProcessRunning = True: Exit Function
    IsProcessRunning = False
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, 0&, PID)
    If hProcess = 0 Then Exit Function
    GetExitCodeProcess hProcess, lngExit
    CloseHandle hProcess
    If lngExit = STILL_ACTIVE Then IsProcessRunning = True
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : KillProcess
' DateTime  : 22/11/2006 23:25
' Author    : Vb System Library
' Purpose   : Termine un processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function KillProcess(ByVal ProcessID As Long) As Long
    Dim hProcess As Long
    ' Ouvre le processus
    hProcess = OpenProcess(PROCESS_TERMINATE, 0, ProcessID)
    If hProcess Then
        ' Termine le processus
        KillProcess = TerminateProcess(hProcess, 0)
        CloseHandle hProcess
    End If
End Function

'---------------------------------------------------------------------------------------
' Procedure : CloseProcess
' DateTime  : 14/04/2007 12:50
' Author    : Vb System Library
' Purpose   : Fermer un processus
' Params    : ProcessID : ID du processus
'           : Delai     : temps d'attente de la fermeture
'           : Force     : indique s'il faut forcer la fermeture
'---------------------------------------------------------------------------------------
Public Function CloseProcess(ByVal ProcessID As Long, Optional Delai As Long = 2000, Optional Force As Boolean = False) As Long
    
    Dim hProcess    As Long
    Dim ExitCode    As Long
    Dim Proc        As New Process
    Dim Windows     As New Windows
    Dim ThID        As Long
    Dim l           As Long
    
    ' Ouvre le processus
    hProcess = OpenProcess(PROCESS_ALL_ACCESS, 0, ProcessID)
    If hProcess Then
        Proc.GetProcessByID ProcessID
        Set Windows = Proc.GetWindows
        'Recuperation et envoi un message de fermeture au thread principal de l'application
        ThID = GetWindowThreadProcessId(Windows.Window(1).Handle, l)
        PostThreadMessageA ThID, WM_QUIT, 0, 0
        
        'on attend la femeture du process
        If WaitForSingleObject(hProcess, Delai) = 0& Then
            'on verifie que celui ci s'est bien arret
            GetExitCodeProcess hProcess, ExitCode
            's 'il est toujours en cours
            If ExitCode = STILL_ACTIVE Then
            'si on a choisi l'option force on le ferme
                If Force = True Then CloseProcess = TerminateProcess(hProcess, 0)
            Else
                CloseProcess = 1
            End If
        Else
            If Force = True Then CloseProcess = TerminateProcess(hProcess, 0)
        End If
        CloseHandle hProcess
    End If
    
End Function

'---------------------------------------------------------------------------------------
' Procedure : SuspendProcess
' DateTime  : 14/04/2007 12:50
' Author    : Vb System Library
' Purpose   : Suspend un processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function SuspendProcess(ByVal ProcessID As Long) As Long
    Dim hProcess As Long
    ' Ouvre le processus
    hProcess = OpenProcess(PROCESS_SUSPEND_RESUME, 0, ProcessID)
    If hProcess Then
        'Suspend le processus
        SuspendProcess = NtSuspendProcess(hProcess)
        CloseHandle hProcess
    End If
End Function
   
'---------------------------------------------------------------------------------------
' Procedure : ResumeProcess
' DateTime  : 14/04/2007 12:50
' Author    : Vb System Library
' Purpose   : relance un processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function ResumeProcess(ByVal ProcessID As Long) As Long
    Dim hProcess As Long
    ' Ouvre le processus
    hProcess = OpenProcess(PROCESS_SUSPEND_RESUME, 0, ProcessID)
    'Relance le processus
    If hProcess Then
        ResumeProcess = NtResumeProcess(hProcess)
        CloseHandle hProcess
    End If
End Function

'---------------------------------------------------------------------------------------
' Procedure : GetProcessFileName
' DateTime  : 23/11/2006 14:42
' Author    : Vb System Library
' Purpose   : Renvoie le chemin complet du processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function GetProcessFileName(ByVal ProcessID As Long) As String
    
    Dim nLen As Long
    
    ' Processus 0
    If ProcessID = 0 Then
        GetProcessFileName = "[System Process]"
    ' Processus 4
    ElseIf ProcessID = 4 Then
        GetProcessFileName = "System"
    Else
        ' On cherche son chemin d'accs complet
        Dim hProcess As Long    'handle du processus
        Dim hModule As Long     'handle du module de l'exe
        Dim Ret As Long         'rsultat
        ' On demande un handle pour le processus
        hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0&, ProcessID)
        ' Si erreur (accs refus)
        If hProcess <> 0 Then
            ' On prformate la chaine
            GetProcessFileName = Space$(MAX_PATH)
            ' On rcupre son nom complet
            nLen = GetModuleFileNameExA(hProcess, 0, GetProcessFileName, MAX_PATH)
            ' On ferme le handle ouvert
            CloseHandle hProcess
            ' On retire le vbNUllChar de fin de chaine
            GetProcessFileName = Left$(GetProcessFileName, nLen)
            FormatPath GetProcessFileName
        Else
            GetProcessFileName = vbNullString
        End If
    End If
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessorTimes
' DateTime  : 23/11/2006 14:35
' Author    : Vb System Library
' Purpose   : Permet d'obtenir les temps processeur, temps noyeau et temps utilisateur d'un processus
'               Attention : Temps renvoys en secondes
' Params    : lPID : ID du processus
'               KernelTime : Temps noyau
'               UserTime : Temps utilisateur
'               ProcessorTime : Temps processeur
'---------------------------------------------------------------------------------------
Public Sub GetProcessorTimes(ByVal lPid As Long, ByRef KernelTime As Double, ByRef UserTime As Double, ByRef ProcessorTime As Double)
    'Le temps processeur (visible dans TaskMgr) est le temps noyau + temps utilisateur
    Dim T0 As FILETIME 'cette variable n'est pas utilise, mais est ncessaire comme argument de l'API
    Dim curTime2 As FILETIME 'Le temps user qui nous intresse
    Dim curTime As FILETIME 'Le temps kernel qui nous intresse
    Dim lProcess As Double 'handle du processus
    Dim lTime2 As Double 'temps que l'on rcupre (user)
    Dim lTime As Double 'temps que l'on rcupre (kernel)
    'obtient le handle du processus
    lProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0, lPid)
    'obtient les temps (T0 ne nous intresse pas)
    If lProcess Then
        If GetProcessTimes(lProcess, T0, T0, curTime, curTime2) Then
            'Renvoit le temps en secondes
            lTime = (curTime.dwLowDateTime + curTime.dwHighDateTime) / 1000
            lTime2 = (curTime2.dwLowDateTime + curTime2.dwHighDateTime) / 1000
            'Ferme le handle ouvert par OpenProcess
            CloseHandle lProcess
            'Affecte la valeur de KernelTime et de UserTime
            KernelTime = lTime
            UserTime = lTime2
            'Le temps processeur est la somme des temps kernel et user
            ProcessorTime = UserTime + KernelTime
        End If
        CloseHandle lProcess
    End If
End Sub


'---------------------------------------------------------------------------------------
' Procedure : GetProcessPriority
' DateTime  : 01/03/2007 22:07
' Author    : Vb System Library
' Purpose   : Renvoie la priorit d'un processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function GetProcessPriority(ProcessID As Long) As Long

    Dim hProcess As Long
    ' Ouvre le processus en lecture
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, 0, ProcessID)
    If hProcess Then
        ' On rcupre sa priorit
        GetProcessPriority = GetPriorityClass(hProcess)
        ' On ferme le pointeur ouvert
        CloseHandle hProcess
    End If
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : SetProcessPriority
' DateTime  : 23/11/2006 14:46
' Author    : Vb System Library
' Purpose   : Defini la priorit d'un processus
' Params    : ProcessID : ID du processus
'               Level : Niveau de priorit
'---------------------------------------------------------------------------------------
Public Function SetProcessPriority(ByVal ProcessID As Long, ByVal Level As Long)
    Dim hProcess As Long
    ' Ouvre en ecriture
    hProcess = OpenProcess(PROCESS_SET_INFORMATION, 0, ProcessID)
    If hProcess Then
        ' On dfinit la priorit
        SetPriorityClass hProcess, Level
        ' On ferme le pointeur ouvert
        CloseHandle hProcess
    End If
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetPriorityName
' DateTime  : 04/04/2007 21:00
' Author    : Vb System Library
' Purpose   : Renvoie le nom de la priorit  partir de son code
' Params    :
'---------------------------------------------------------------------------------------
Public Function GetPriorityName(ByVal Priority As Long) As String
    Select Case Priority
        Case IDLE_PRIORITY_CLASS:           GetPriorityName = "IDLE"
        Case BELOW_NORMAL_PRIORITY_CLASS:   GetPriorityName = "BELOW_NORMAL"
        Case NORMAL_PRIORITY_CLASS:         GetPriorityName = "NORMAL"
        Case ABOVE_NORMAL_PRIORITY_CLASS:   GetPriorityName = "ABOVE_NORMAL"
        Case HIGH_PRIORITY_CLASS:           GetPriorityName = "HIGH"
        Case REALTIME_PRIORITY_CLASS:       GetPriorityName = "REALTIME"
    End Select
End Function
Public Function GetPriorityValue(ByVal Priority As ProcessPriority) As Long
    Select Case Priority
        Case [Idle]:            GetPriorityValue = IDLE_PRIORITY_CLASS
        Case [Below Normal]:    GetPriorityValue = BELOW_NORMAL_PRIORITY_CLASS
        Case [Normal]:          GetPriorityValue = NORMAL_PRIORITY_CLASS
        Case [Above Normal]:    GetPriorityValue = ABOVE_NORMAL_PRIORITY_CLASS
        Case [High]:            GetPriorityValue = HIGH_PRIORITY_CLASS
        Case [Real Time]:       GetPriorityValue = REALTIME_PRIORITY_CLASS
    End Select
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessMemoryUsage
' DateTime  : 23/11/2006 15:34
' Author    : Vb System Library
' Purpose   : Renvoie une structure PROCESS_MEMORY_COUNTERS contenant toutes
'               les infos sur l'utilisation mmoire du processus
' Params    : ProcessID : ID du processus
'---------------------------------------------------------------------------------------
Public Function GetProcessMemoryInfos(ByVal ProcessID As Long) As PROCESS_MEMORY_COUNTERS
    Dim hProcess As Long
    Dim pmc As PROCESS_MEMORY_COUNTERS
    ' Rcupre le handle du processus  partir de son identifiant
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0, ProcessID)
    If hProcess Then
        ' On prpare pmc
        pmc.cb = LenB(pmc)
        ' On rcupre les infos du processus
        GetProcessMemoryInfo hProcess, pmc, pmc.cb
        CloseHandle hProcess
        GetProcessMemoryInfos = pmc
    End If
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetProcessUserName
' DateTime  : 23/11/2006 15:41
' Author    : Vb System Library
' Purpose   : Renvoie le nom d'utilisateur qui a ouvert le processus
' Params    : Pid : ID du processus
'---------------------------------------------------------------------------------------
Public Function GetProcessUserName(ByVal PID As Long) As String

    Dim hToken              As Long
    Dim hProcess            As Long
    Dim cbBuff              As Long
    Dim TG                  As TOKEN_GROUPS
    Dim UserName            As String
    Dim DomainName          As String
    Dim UserNameLenght      As Long
    Dim DomainNameLenght    As Long
    Dim peUse               As Long
    Dim ppsidGroup          As Long
    
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0, PID)
    
    If hProcess Then
        If OpenProcessToken(hProcess, &H8, hToken) Then
            GetTokenInformation hToken, TOKEN_USER, TG, 0, cbBuff
            If GetTokenInformation(hToken, TOKEN_USER, TG, cbBuff, 0) Then
                CloseHandle hToken
                UserNameLenght = 255
                UserName = Space$(UserNameLenght)
                DomainName = UserName 'Space$(255)
                DomainNameLenght = UserNameLenght
                LookupAccountSid vbNullString, TG.GroupCount, UserName, UserNameLenght, DomainName, DomainNameLenght, peUse
                GetProcessUserName = Left$(UserName, UserNameLenght)
            End If
        Else
            CloseHandle hProcess
            hProcess = OpenProcess(PROCESS_READ_CONTROL, 0, PID)
            If hProcess Then
                If GetSecurityInfo(hProcess, SE_KERNEL_OBJECT, GROUP_SECURITY_INFORMATION, 0, ppsidGroup, 0, 0, 0) = 0 Then
                    UserNameLenght = 255
                    UserName = Space$(UserNameLenght)
                    DomainName = UserName
                    DomainNameLenght = UserNameLenght
                    LookupAccountSid vbNullString, ppsidGroup, UserName, UserNameLenght, DomainName, DomainNameLenght, peUse
                    GetProcessUserName = Left$(UserName, UserNameLenght)
                End If
                CloseHandle hProcess
            End If
        End If
        CloseHandle hProcess
    End If

End Function


'---------------------------------------------------------------------------------------
' Procedure : GetParentProcessId
' DateTime  : 03/12/2006 15:58
' Author    : Vb System Library
' Purpose   : Renvoie l'ID du processus parent
' Params    : Pid : ID du processus
'---------------------------------------------------------------------------------------
Public Function GetParentProcessID(ByVal PID As Long) As Long
    Dim hProcess    As Long
    Dim Pbi         As PROCESS_BASIC_INFORMATION
    Dim Ret         As Long
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION, 0&, PID)
    If hProcess Then
        NtQueryInformationProcess hProcess, 0&, VarPtr(Pbi), 24&, Ret
        GetParentProcessID = Pbi.InheritedFromUniqueProcessId
        CloseHandle hProcess
    End If
End Function


'---------------------------------------------------------------------------------------
' Procedure : RetrieveMemRegions
' DateTime  : 07/12/2006 19:48
' Author    : Vb System Library
' Purpose   : Rcupre les diffrentes zones mmoire d'un processus utilises dans
'               la mmoire virtuelle
' Params    :   lPid : ID du processus
'               lBaseAdress() : tableau de long recevant les adresses des zones mmoire
'               lRegionSize() : tableau de long recevant la taille des zones mmoire
'---------------------------------------------------------------------------------------
Public Sub RetrieveProcessMemoryRegions(ByVal lPid As Long, ByRef lBaseAdress() As Long, ByRef lRegionSize() As Long)
    Dim lHandle As Long
    Dim lPosMem As Long
    Dim lRet As Long
    Dim lLenMBI As Long
    Dim mbi As MEMORY_BASIC_INFORMATION
    Dim si As SYSTEM_INFO

    'initialise les tableaux
    ReDim lBaseAdress(0)
    ReDim lRegionSize(0)
                    
    'obtient le handle du processus
    lHandle = OpenProcess(PROCESS_ALL_ACCESS, False, lPid)

    lLenMBI = Len(mbi)  'taille de la structure
    GetSystemInfo VarPtr(si)    'obtient les infos sur les adresses de dbut et de fin de la plage mmoire maximum
    
    lPosMem = si.lpMinimumApplicationAddress  'adresse la plus petite ==> part de l
    
    Do While lPosMem < si.lpMaximumApplicationAddress 'tant que l'adresse est infrieure  l'adresse maximale
        mbi.RegionSize = 0
        
        'obtient les infos sur les rgions mmoire du processus dfinit par son handle hProcess
        lRet = VirtualQueryEx(lHandle, ByVal lPosMem, mbi, lLenMBI)
        
        If lRet = lLenMBI Then
        
            If (mbi.lType = MEM_PRIVATE) And (mbi.State = MEM_COMMIT) Then
                'alors utilis par le processus
                
                If mbi.RegionSize > 0 Then
                    'rgion non nulle, alors on la stocke dans les tableaux rsultats
                    'les redimensionne
                    ReDim Preserve lBaseAdress(UBound(lBaseAdress) + 1)
                    ReDim Preserve lRegionSize(UBound(lRegionSize) + 1)
                    'stocke  la fin
                    lRegionSize(UBound(lRegionSize)) = mbi.RegionSize
                    lBaseAdress(UBound(lBaseAdress)) = mbi.BaseAddress
                End If
                
            End If
            'continue la recherche des rgions (ajoute la taille de la rgion  l'adresse de dpart ==> donne la prochaine adresse de dpart)
            
            On Error GoTo ErrCapacityGestion  'dpassement de capacit pour la dernire adresse+regiosize
            
            lPosMem = mbi.BaseAddress + mbi.RegionSize    'fait l'ajout
        
        Else
            'recherche termine
            Exit Do
        End If
    Loop
    
ErrCapacityGestion:
   CloseHandle lHandle 'ferme le handle du processus
End Sub

' Formate le chemin d'accs aux fichiers des processus
Private Sub FormatPath(ByRef Path As String)
    Dim Files As New clsFileSystem
    If Left$(Path, 1) = "\" Then
        If Left$(Path, 2) = "\?" Then
            Path = Replace(Path, "\??\", "")
        ElseIf Left$(Path, 2) = "\S" Then
            Path = Replace(Path, "\SystemRoot", Files.GetSpecialFolder(CSIDL_WINDOWS))
        End If
    End If
    Set Files = Nothing
End Sub


'---------------------------------------------------------------------------------------
' Procedure : EnableDebugPrivileges
' DateTime  : 22/11/2006 23:05
' Author    : Vb System Library
' Purpose   : donne le privilege Debug  l'application
'             (permet d'accder  tout les processus avec OpenProcess)
'---------------------------------------------------------------------------------------
Public Sub EnableDebugPrivileges()

    Dim PID As Long
    'obtient l'id de ce process
    PID = GetCurrentProcessId
    'change les droits de cette application
    SetPrivilege PID, SE_DEBUG_NAME, True

End Sub


'---------------------------------------------------------------------------------------
' Procedure : SetPrivilege
' DateTime  : 04/04/2007 20:35
' Author    : Vb System Library
' Purpose   : Donne un niveau de privilge  un processus
' Params    : ProcessID : ID du processus
'               seName : Nom du privilege
'---------------------------------------------------------------------------------------
Public Function SetPrivilege(ByVal ProcessID As Long, ByVal seName As String, ByVal Status As Boolean) As Boolean
    
    Dim hProcess            As Long
    Dim hProcessToken       As Long
    Dim Ret                 As Long
    Dim p_lngToken          As Long
    Dim p_lngBufferLen      As Long
    Dim p_typLUID           As LUID
    Dim p_typTokenPriv      As TOKEN_PRIVILEGES
    Dim p_typPrevTokenPriv  As TOKEN_PRIVILEGES
    
    hProcess = OpenProcess(PROCESS_ALL_ACCESS, False, ProcessID)
    If hProcess Then
        hProcessToken = OpenProcessToken(hProcess, TOKEN_ADJUST_PRIVILEGES Or TOKEN_QUERY, p_lngToken)
        If hProcessToken Then
            Ret = LookupPrivilegeValue(vbNullString, seName, p_typLUID)
            If Ret Then
                p_typTokenPriv.PrivilegeCount = 1
                p_typTokenPriv.Privileges.Attributes = GetStatusLong(Status)
                p_typTokenPriv.Privileges.pLuid = p_typLUID
                SetPrivilege = (AdjustTokenPrivileges(p_lngToken, False, p_typTokenPriv, Len(p_typPrevTokenPriv), p_typPrevTokenPriv, p_lngBufferLen) = 1)
            End If
        End If
    End If
    
End Function


'---------------------------------------------------------------------------------------
' Procedure : GetFullPrivilegeName
' DateTime  : 04/04/2007 20:36
' Author    : Vb System Library
' Purpose   : Renvoie le nom du privilge
' Params    : seName : Nom du privilege
'---------------------------------------------------------------------------------------
Public Function GetFullPrivilegeName(ByVal sName As String) As String

    Select Case UCase(sName)
        Case UCase("Audit"):                GetFullPrivilegeName = SE_AUDIT_NAME
        Case UCase("Backup"):               GetFullPrivilegeName = SE_BACKUP_NAME
        Case UCase("ChangeNotify"):         GetFullPrivilegeName = SE_CHANGE_NOTIFY_NAME
        Case UCase("CreatePagefile"):       GetFullPrivilegeName = SE_CREATE_PAGEFILE_NAME
        Case UCase("CreatePermanent"):      GetFullPrivilegeName = SE_CREATE_PERMANENT_NAME
        Case UCase("CreateToken"):          GetFullPrivilegeName = SE_CREATE_TOKEN_NAME
        Case UCase("Debug"):                GetFullPrivilegeName = SE_DEBUG_NAME
        Case UCase("RemoteShutdown"):       GetFullPrivilegeName = SE_REMOTE_SHUTDOWN_NAME
        Case UCase("ProfileSingleProcess"): GetFullPrivilegeName = SE_PROF_SINGLE_PROCESS_NAME
        Case UCase("Restore"):              GetFullPrivilegeName = SE_RESTORE_NAME
        Case UCase("Security"):             GetFullPrivilegeName = SE_SECURITY_NAME
        Case UCase("Shutdown"):             GetFullPrivilegeName = SE_SHUTDOWN_NAME
        Case UCase("SystemEnvironment"):    GetFullPrivilegeName = SE_SYSTEM_ENVIRONMENT_NAME
        Case UCase("SystemProfile"):        GetFullPrivilegeName = SE_SYSTEM_PROFILE_NAME
        Case UCase("Systemtime"):           GetFullPrivilegeName = SE_SYSTEMTIME_NAME
        Case UCase("TakeOwnership"):        GetFullPrivilegeName = SE_TAKE_OWNERSHIP_NAME
        Case UCase("Tcb"):                  GetFullPrivilegeName = SE_TCB_NAME
        Case UCase("ManageVolume"):         GetFullPrivilegeName = SE_MANAGE_VOLUME_NAME
        Case UCase("IncreaseBasePriority"): GetFullPrivilegeName = SE_INC_BASE_PRIORITY_NAME
        Case UCase("IncreaseQuota"):        GetFullPrivilegeName = SE_INCREASE_QUOTA_NAME
        Case UCase("LockMemory"):           GetFullPrivilegeName = SE_LOCK_MEMORY_NAME
        Case UCase("LoadDriver"):           GetFullPrivilegeName = SE_LOAD_DRIVER_NAME
        Case UCase("MachineAccount"):       GetFullPrivilegeName = SE_MACHINE_ACCOUNT_NAME
    End Select
    
End Function
Private Function GetStatusLong(ByVal Status As Boolean) As Long
    If Status = True Then GetStatusLong = SE_PRIVILEGE_ENABLED Else GetStatusLong = SE_PRIVILEGE_DISBALED
End Function

Public Function GetCoreListFromMask(Mask As Long, CoreList() As Byte) As Integer
    
    Dim MaxCore     As Byte
    Dim CoreCount   As Byte
    Dim i           As Byte
    Dim Value       As Long
    
    MaxCore = 16
    
    For i = 0 To MaxCore - 1
        Value = (2 ^ i)
        If Mask < Value Then Exit For
        If Mask And Value Then
            ReDim Preserve CoreList(CoreCount)
            CoreList(CoreCount) = i
            CoreCount = CoreCount + 1
        End If
    Next i
    
    GetCoreListFromMask = CoreCount
    
End Function

Public Function GetMaskFromCoreList(CoreList() As Byte) As Long
    
    Dim i As Byte
    
    On Error GoTo ErrInvalide
    
    For i = LBound(CoreList) To UBound(CoreList)
        GetMaskFromCoreList = GetMaskFromCoreList + 2 ^ CoreList(i)
    Next i
    
    Exit Function
    
ErrInvalide:
    GetMaskFromCoreList = -1
End Function


Public Function CoreListToString(CoreList() As Byte) As String
    
    Dim i As Byte
    
    On Error GoTo ErrInvalid
    
    For i = LBound(CoreList) To UBound(CoreList)
        CoreListToString = CoreListToString & CoreList(i) & ","
    Next i
    
    CoreListToString = Left(CoreListToString, Len(CoreListToString) - 1)
    Exit Function
    
ErrInvalid:
    CoreListToString = ""
End Function

Public Function CoreListToByteArray(CoreList As String) As Byte()

    Dim s() As String
    Dim b() As Byte
    Dim i   As Integer
    
    If CoreList = "" Then Exit Function
    
    s = Split(CoreList, ",")
    ReDim b(UBound(s))
    
    For i = 0 To UBound(b)
        b(i) = CByte(s(i))
    Next i
    
    Erase s
    CoreListToByteArray = b
    
End Function


Private Sub GetProcessFullInformation(ByRef Proc As SYSTEM_PROCESSES, ByRef ProcInfo As ProcessInfo, ByRef ThreadInfos() As ThreadInfo)

    Dim hProcess    As Long
    Dim Ret         As Long
    Dim Pbi         As PROCESS_BASIC_INFORMATION
    Dim pp          As PROCESS_PARAMETERS
    Dim m_PEB       As peb
    Dim i           As Long
    
    With ProcInfo
        .ProcessID = Proc.ProcessID
        .ThreadCount = Proc.ThreadCount
        .PriorityClass = Proc.BasePriority
        .ParentProcessID = Proc.InheritedFromProcessId
        .HandleCount = Proc.HandleCount
        .PagefileUsage = Proc.VmCounters.WorkingSetSize
        .UserName = GetProcessUserName(.ProcessID)
        .ProcessName = Space$(Proc.ProcessName.Length \ 2)
        CopyMemory ByVal StrPtr(.ProcessName), ByVal Proc.ProcessName.Buffer, Proc.ProcessName.Length
    End With
    
    ' Ouvre le processus et demande ses infos
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0&, Proc.ProcessID)
    If hProcess Then
        NtQueryInformationProcess hProcess, 0&, VarPtr(Pbi), 24&, Ret
        With ProcInfo
            ReadProcessMemory hProcess, Pbi.PEBBaseAddress, VarPtr(m_PEB), Len(m_PEB), Ret
            ReadProcessMemory hProcess, m_PEB.ProcessParameters, VarPtr(pp), Len(pp), Ret
            .ImageFile = Space$(pp.ImageFile.Length \ 2)
            ReadProcessMemory hProcess, pp.ImageFile.Buffer, StrPtr(.ImageFile), pp.ImageFile.Length, Ret
            FormatPath .ImageFile
            .CommandLine = Space$(pp.CommandLine.Length \ 2)
            ReadProcessMemory hProcess, pp.CommandLine.Buffer, StrPtr(.CommandLine), pp.CommandLine.Length, Ret
            .CurrentDirectory = Space$(pp.CurrentDirectoryName.Length \ 2)
            ReadProcessMemory hProcess, pp.CurrentDirectoryName.Buffer, StrPtr(.CurrentDirectory), pp.CurrentDirectoryName.Length, Ret
        End With
        CloseHandle hProcess
    End If

End Sub

Private Sub GetProcessBasicInformation(ByRef Proc As SYSTEM_PROCESSES, ByRef ProcInfo As ProcessBasicInformations)

    Dim hProcess    As Long
    Dim Ret         As Long
    Dim Pbi         As PROCESS_BASIC_INFORMATION
    Dim pp          As PROCESS_PARAMETERS
    Dim m_PEB       As peb
    
    With ProcInfo
        .ProcessID = Proc.ProcessID
        .ParentProcessID = Proc.InheritedFromProcessId
        .UserName = GetProcessUserName(.ProcessID)
        .ProcessName = Space$(Proc.ProcessName.Length \ 2)
        CopyMemory ByVal StrPtr(.ProcessName), ByVal Proc.ProcessName.Buffer, Proc.ProcessName.Length
    End With
    
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION Or PROCESS_VM_READ, 0&, Proc.ProcessID)
    If hProcess Then
        NtQueryInformationProcess hProcess, 0&, VarPtr(Pbi), 24&, Ret
        With ProcInfo
            ReadProcessMemory hProcess, Pbi.PEBBaseAddress, VarPtr(m_PEB), Len(m_PEB), Ret
            ReadProcessMemory hProcess, m_PEB.ProcessParameters, VarPtr(pp), Len(pp), Ret
            .PEBAdress = Pbi.PEBBaseAddress
            .ImageFile = Space$(pp.ImageFile.Length \ 2)
            ReadProcessMemory hProcess, pp.ImageFile.Buffer, StrPtr(.ImageFile), pp.ImageFile.Length, Ret
            FormatPath .ImageFile
            .CommandLine = Space$(pp.CommandLine.Length \ 2)
            ReadProcessMemory hProcess, pp.CommandLine.Buffer, StrPtr(.CommandLine), pp.CommandLine.Length, Ret
            .CurrentDirectory = Space$(pp.CurrentDirectoryName.Length \ 2)
            ReadProcessMemory hProcess, pp.CurrentDirectoryName.Buffer, StrPtr(.CurrentDirectory), pp.CurrentDirectoryName.Length, Ret
        End With
        CloseHandle hProcess
    End If

End Sub

Private Sub GetThreadFullInformation(ByRef Proc As SYSTEM_PROCESSES, ByRef ThreadInfos() As ThreadInfo)
    
    Dim i As Long
    If Proc.ThreadCount = 0 Then Exit Sub
    ReDim ThreadInfos(Proc.ThreadCount - 1)
    
    For i = 0 To Proc.ThreadCount - 1
        With ThreadInfos(i)
            .ID = Proc.Threads(i).ClientId.UniqueThread
            .BasePriority = Proc.Threads(i).BasePriority
            .Priority = Proc.Threads(i).Priority
            .StartAddress = Proc.Threads(i).StartAddress
            .ParentProcessID = Proc.ProcessID
            .State = Proc.Threads(i).State
            .ContextSwitchCount = Proc.Threads(i).ContextSwitchCount
            .CreateTime = Proc.Threads(i).CreateTime
            .KernelTime = Proc.Threads(i).KernelTime
            .UserTime = Proc.Threads(i).UserTime
            .WaitTime = Proc.Threads(i).WaitTime
        End With
    Next i

End Sub

